//--------------------------------------------------------------------
//		Timescale
//		Means that if you do #1 in the initial block of your
//		testbench, time is advanced by 1ns instead of 1ps
//--------------------------------------------------------------------
`timescale 1ns / 1ps
//--------------------------------------------------------------------

//--------------------------------------------------------------------
//		The Lab4Basic Testbench
//		Try to figure out what this does without using ModelSim
//		Run this in ModelSim to verify its functionality
//--------------------------------------------------------------------
module Lab4BasicTestbench();

	//----------------------------------------------------------------
	//		Parameters
	//----------------------------------------------------------------
	parameter ResetValue	= 2'b01;
	parameter HalfCycle		= 5;			//Half of the Clock Period is 5 ns
	//----------------------------------------------------------------

	//----------------------------------------------------------------
	//		Constants
	//----------------------------------------------------------------
	localparam Cycle		= 2*HalfCycle;	//The length of the entire Clock Period
	//----------------------------------------------------------------

	//----------------------------------------------------------------
	//		Declare Signals to connect to Lab4Basic
	//		Note that several signals are declared as a 'reg'
	//		This is because we set their values inside of an 'initial'
	//		block
	//----------------------------------------------------------------
	reg					Clock;
	reg					Reset;
	
	reg					In;
	wire				Out;
	
	integer				i;
	//----------------------------------------------------------------

	//----------------------------------------------------------------
	//		Clock Generator
	//----------------------------------------------------------------
	initial Clock	= 0;				//Clock is at 0 at time 0 of the simulation
	always #(HalfCycle) Clock = ~Clock;	//Every half of a Clock period, flip the Clock
	//----------------------------------------------------------------
	
	//----------------------------------------------------------------
	//		Lab4Basic DUT Instantiation
	//----------------------------------------------------------------
	Lab4Basic	#	(	.ResetValue	(ResetValue))
				DUT	(	.Clock		(Clock),
						.Reset		(Reset),
						.In			(In),
						.Out		(Out));
	//----------------------------------------------------------------

	//----------------------------------------------------------------
	//		Test Stimulus
	//		This is where you get to change the inputs going into Lab4Basic
	//		You will need to figure out what this does
	//----------------------------------------------------------------
	initial begin
		In 		= 0;
		Reset 	= 0;
		#(Cycle);
		
		//Issue the Reset
		Reset	= 1;	//Set Reset high
		#(Cycle);		//Wait one cycle
		Reset	= 0;	//Bring Reset low

		In 		= 1;
		#(Cycle);
		In		= 0;
		#(2*Cycle);		//This means wait a period of two cycles
		In		= 1;
		#(Cycle);
		In		= 0;
		#(Cycle);
		In		= 1;
		#(3*Cycle);
		In		= 0;
		#(Cycle);
		In		= 1;
		#(Cycle);
		In		= 0;
		
		//Example of a for loop in verilog. This just flips the In signal every cycle
		for (i = 0; i < 15; i = i + 1) begin
			#(Cycle);
			In 		= ~In;
		end
		
		$display("INFO[%m @ %t]: Test sequence completed", $time);
	end
	//----------------------------------------------------------------
endmodule
